/************************************************************************
 ========================================================================
 CORAL 
 (c)  Copyright R. Ramakrishnan and The CORAL Group, 
 University of Wisconsin at Madison.
 (1992) All Rights Reserved.
 Version 0.1
 ========================================================================



 ------------------------------------------------------------------------
 CORAL Version 0.1
 RESEARCH SOFTWARE DISCLAIMER -------------------------------------------
 ------------------------------------------------------------------------

    As unestablished, research software, this program is provided free of 
    charge on an "as is" basis without warranty of any kind, either 
    express or implied.  Acceptance and use of this program constitutes 
    the user's understanding that (s)he will have no recourse for any 
    actual or consequential damages, including, but not limited to, 
    lost profits or savings, arising out of the use of or inability to 
    use this program.  

 ------------------------------------------------------------------------
 USER AGREEMENT ---------------------------------------------------------
 ------------------------------------------------------------------------

     BY ACCEPTANCE AND USE OF THIS EXPERIMENTAL PROGRAM
     THE USER AGREES TO THE FOLLOWING:

     a.  This program is provided free of charge for the user's personal, 
	 non-commercial, experimental use.

     b.  All title, ownership and rights to this program and any copies 
         remain with the copyright holder, irrespective of the ownership 
	 of the media on which the program resides.

     c.  The user is permitted to create derivative works to this program.  
         However, all copies of the program and its derivative works must
         contain the CORAL copyright notice, the UNESTABLISHED SOFTWARE 
         DISCLAIMER and this USER AGREEMENT.

     d.  The user understands and agrees that this program and any 
         derivative works are to be used solely for experimental purposes 
	 and are not to be sold or commercially exploited in any manner 
	 WITHOUT EXPRESS WRITTEN PERMISSION.

     e.  We request that the user supply us with a copy of any changes, 
         enhancements, or derivative works which the user may create,
	 with the user's permission to redistribute it.
	 Copies of such material should be sent to:  CORAL@CS.WISC.EDU

-------------------------------------------------------------------------
*************************************************************************/

#ifndef PERSISTENT_REL_H
#define PERSISTENT_REL_H
#include "generic-sm.h"
#include "generic-rel.h"
#include "persistent-index.h"

#define Scanning_Persistent 1

class PersistentRel : public StorageRelation {
  friend class PersistentIndex;
  friend class PersistentIndexSet;
 private:

  OID            oid;           // OID of FileDescStruct of the relation
  FileDescStruct fd;		// File descriptor

  LOADDESC       *lddesc;       // bulk load descriptor

  PersistentIndexSet index_set;

// Need representation of field types

  void insertObject(ArgList &arglist) ;
  Tuple *fileScanGetNext(TupleIterator &, int delete_flag = 0) ;
  Tuple *indexScanGetNext(TupleIterator&, int, int delete_flag = 0);

  // Returns position where an index entry can be made
  //int canAddIndex() { return fd.canAddIndex() ;} ;
  Tuple *checkTuple(USERDESC *, TupleIterator&) ;

 public:
  PersistentRel(Name name, int arity, int& error);
  PersistentRel(Name name, int arity, int& error, OID*);
  PersistentRel(Name name, int arity, int& error, FileDescStruct *);

  //virtual RelationKind find_r_kind() { return COR_R_EXODUS;}
  // NEW_CHNG:PS

  virtual void release(TupleIterator& iterator) ;
  virtual void print_name(FILE *file);
  virtual void print_facts(FILE *file, ArgList *arglist = NULL) const ;

  virtual void freeMark(RMark *, int hide_em=0);
  virtual void free_allMarks();

  virtual BindEnv *get_next(TupleIterator&);
  virtual Tuple  *get_next_tuple(TupleIterator&) ;
  virtual int insert_raw(Tuple *tuple);

  virtual int insert_tuple(Tuple *tuple);
// maybe this can use the ::insert_new defined in generic-rel ????
 /** virtual int insert_new(ArgList& args, BindEnv* env,
			 BindEnv *dont_rename_env = NULL,
			 Tuple *parent_goal = NULL,
			 int dont_rename = 0); 
  ***/

  virtual GenericIndexSet *get_index_set() { return &index_set;}
  virtual GenericIndex *create_index_object(BitVector *bv)
    { return new PersistentIndex(this, bv);}
  virtual GenericIndex *create_index_object(ArgList *, int, ArgList *) 
    { return NULL;}

  virtual int tuple_delete(Tuple *tuple);
  // Empty tuples out of the relation
  virtual void empty_relation(int delete_tuples = 0); 
  void initialize_iterator( TupleIterator& );
  virtual int is_subsumed(Tuple *tuple, RMark *startmark= NULL, 
			RMark *endmark = NULL);

  int start_bulk_load();
  int end_bulk_load();
  void flush_to_disk();

};

#endif 


